@@ -17,14 +17,11 @@
 
 from scivae import VAE
 from sciviso import Heatmap, Histogram, Scatterplot
-import seaborn as sns
 from scipy import stats
 from sciutil import SciUtil
 import matplotlib.pyplot as plt
 from sklearn.preprocessing import MinMaxScaler
 from statsmodels.stats.multitest import multipletests
-import matplotlib
-from matplotlib import rcParams
 import pandas as pd
 import os
 
@@ -35,46 +32,73 @@ This class is build when VAE is created and allows for the plots to be generated
 
 class Vis:
 
-    def __init__(self, vae: VAE, sciutil: SciUtil, colours: list):
+    def __init__(self, vae: VAE, sciutil: SciUtil, colours: list, config=None):
         self.vae = vae
         self.u = sciutil
-        self.colours = colours if colours else ['#483873', '#1BD8A6', '#B117B7', '#AAC7E2', '#FFC107', '#016957', '#9785C0',
-             '#D09139', '#338A03', '#FF69A1', '#5930B1', '#FFE884', '#35B567', '#1E88E5',
-             '#ACAD60', '#A2FFB4', '#B618F5', '#854A9C']
+        self.colours = colours if colours else ['#483873', '#1BD8A6', '#B117B7', '#AAC7E2', '#FFC107', '#016957',
+                                                '#9785C0', '#D09139', '#338A03', '#FF69A1', '#5930B1', '#FFE884',
+                                                '#35B567', '#1E88E5', '#ACAD60', '#A2FFB4', '#B618F5', '#854A9C']
         self.df = None
-        self.label_font_size = 8
-        self.title_font_size = 10
-        self.title_font_weight = 700
-        self.fig_size = (2, 2)
-        sns.set_style('ticks')
-        self.alpha = 0.6
-        rcParams['figure.figsize'] = self.fig_size
-        sns.set_context("paper", rc={"font.size": self.label_font_size, "axes.titlesize": self.title_font_size,
-                                     "axes.labelsize": self.label_font_size,
-                                     'figure.figsize': self.fig_size})
-        self.style = {}
+        self.base_config = {
+            'font': {'family': 'normal', 'size': 6},
+            'label_font_size': 7,
+            'title_font_size': 10,
+            'style': "ticks",
+            'figsize': (3, 3),
+            'alpha': 0.6
+        }
+        self.base_config = self.__merge_config(self.base_config, config)
 
-    def plot_input_distribution(self, df: pd.DataFrame, row_id="", columns=None, nbins=20, ymax=None, output_dir="",
-                                c="grey", fig_type='svg', title="", show_plt=False, save_fig=True):
+    def __merge_config(self, default_config, user_config):
+        # Add in default params from our base
+        for c in self.base_config:
+            if not default_config.get(c):
+                default_config[c] = self.base_config[c]
+        if user_config:
+            for c in default_config:
+                # Update any of the config if the user has supplied it
+                user_config[c] = user_config.get(c) if user_config.get(c) is not None else default_config.get(c)
+            return user_config
+        return default_config
+
+    def plot_input_distribution(self, df: pd.DataFrame, row_id="", columns=None, output_dir="",
+                                fig_type='svg', title="Hist", show_plt=False, save_fig=True, user_config=None):
         columns = columns if columns is not None else [c for c in df.columns if c != row_id]
-
+        config = {
+            'figsize': (4, 4),
+            'nbins': 20,
+            'c': "grey",
+            'max_y': None,
+            'title': title
+        }
+        config = self.__merge_config(config, user_config)
         for column in columns:
-            set_style(self.fig_size, self.label_font_size)
+            set_style()
             try:
-                hist = Histogram(df, column, title=column.replace('_', ' '), bins=nbins, colour=c, max_y=ymax)
-                hist.load_style(self.style)
+                hist = Histogram(df, column, title=column.replace('_', ' '), config=config)
                 hist.plot()
                 if save_fig:
-                    plt.savefig(os.path.join(output_dir, f'{title}_Hist_{column}.{fig_type}'))
+                    plt.savefig(os.path.join(output_dir, f'{title}_{column}.{fig_type}'))
                 if show_plt:
                     plt.show()
             except:
                 self.u.warn_p(["plot_input_distribution: Unable to plot histogram for column:", column])
 
-    def plot_feature_correlation(self, df: pd.DataFrame, row_id="", columns=None, output_dir="", vmin=-1, vmax=1, fig_type='svg',
-                                cluster_cols=True, cluster_rows=True, print_vals=True, cmap='RdBu_r',
-                                 title="RCM heatmap of feature correlation", show_plt=False, save_fig=True):
+    def plot_feature_correlation(self, df: pd.DataFrame, row_id="", columns=None, output_dir="",
+                                 fig_type='svg', print_vals=True, title="RCM heatmap of feature correlation",
+                                 show_plt=False, save_fig=True, user_config=None):
         heatmap_df = pd.DataFrame()
+        config = {
+            'figsize': (4, 4),
+            'cluster_cols': True,
+            'cluster_rows': True,
+            'vmin': -1,
+            'vmax': 1,
+            'cmap': 'RdBu_r',
+            'title': title
+        }
+        config = self.__merge_config(config, user_config)
+
         lbls = []
         padj_cols = []
         columns = columns if columns is not None else [c for c in df.columns if c != row_id]
@@ -90,15 +114,12 @@ class Vis:
             lbls.append(f'{feature_1}')
             heatmap_df[f'{feature_1}'] = vals
             reg, padj, a, b = multipletests(pvals, alpha=0.1, method='fdr_bh', returnsorted=False)
-            heatmap_df[f'{feature_1} padj'] = [float(f'{p:.3f}') for p in padj] #heatmap_df[f'Node {i + 1} padj'] = [float(f'rho:{vals[ip]:.3f} p:{p:.3f}') for ip, p in enumerate(padj)]
+            heatmap_df[f'{feature_1} padj'] = [float(f'{p:.3f}') for p in padj]
+            # heatmap_df[f'Node {i + 1} padj'] = [float(f'rho:{vals[ip]:.3f} p:{p:.3f}') for ip, p in enumerate(padj)]
             padj_cols.append(f'{feature_1} padj')
         heatmap_df['labels'] = lbls
-        annot_values = heatmap_df[padj_cols].values
-        heatmap = Heatmap(heatmap_df, lbls, 'labels',
-                          title=f'{title}',
-                          cluster_cols=cluster_cols, cluster_rows=cluster_rows,
-                          vmin=vmin, vmax=vmax, cmap=cmap, figsize=(4, 4)
-                          )
+        # annot_values = heatmap_df[padj_cols].values
+        heatmap = Heatmap(heatmap_df, lbls, 'labels', config=config)
         g = heatmap.plot()
         if save_fig:
             plt.savefig(os.path.join(output_dir, f'Heatmap_{title.replace(" ", "")}.{fig_type}'))
@@ -107,14 +128,23 @@ class Vis:
         return heatmap_df
 
     def plot_top_values_by_rank(self, df: pd.DataFrame, rank_colums: list, vis_columns: list, id_col: str,
-                                num_values=10, cmap='RdBu_r',
-                                fig_type='svg', cluster_cols=True, cluster_rows=True, output_dir="",
-                                title="Ranked top values", show_plt=False, save_fig=True):
+                                num_values=10, fig_type='svg', output_dir="", title="Ranked top values",
+                                show_plt=False, save_fig=True, user_config=None):
         """
         Basically, for each of the rank columns we want to plot the values (from value filter) and the
         top and bottom of the ranks. We plot the vis columns for each of these. Note the vis columns
         should all have the same range.
         """
+        config = {
+            'figsize': (4, 4),
+            'cluster_cols': True,
+            'cluster_rows': True,
+            'vmin': -1,
+            'vmax': 1,
+            'cmap': 'RdBu_r',
+            'title': title,
+        }
+        config = self.__merge_config(config, user_config)
         rank_colours = []
         order_colours = []
         value_cols = []
@@ -155,12 +185,9 @@ class Vis:
 
             heatmap_df[v_c] = values
             value_cols.append(v_c)
-
         heatmap_df[id_col] = gs  # This should be the same irrespective since the
-
         heatmap = Heatmap(heatmap_df, value_cols, id_col, row_colours=[rank_colours, order_colours], title=title,
-                          cluster_cols=cluster_cols, cluster_rows=cluster_rows, cmap=cmap)
-        set_style(self.fig_size, self.label_font_size)
+                          config=config)
         heatmap.plot()
         if save_fig:
             plt.savefig(os.path.join(output_dir, f'Heatmap_{title.replace(" ", "_")}.{fig_type}'))
@@ -168,10 +195,19 @@ class Vis:
             plt.show()
         plt.show()
 
-    def plot_node_correlation(self, output_dir="", vmin=-1, vmax=1, fig_type='svg',
-                                cluster_cols=True, cluster_rows=True, print_vals=True, cmap='RdBu_r',
-                                 title="RCM heatmap of node correlation", show_plt=False, save_fig=True):
+    def plot_node_correlation(self, output_dir="", fig_type='svg', print_vals=True, title="RCM heatmap of node cor.",
+                              show_plt=False, save_fig=True, user_config=None):
         heatmap_df = pd.DataFrame()
+        config = {
+            'figsize': (4, 4),
+            'cluster_cols': True,
+            'cluster_rows': True,
+            'vmin': -1,
+            'vmax': 1,
+            'cmap': 'RdBu_r',
+            'title': title,
+        }
+        config = self.__merge_config(config, user_config)
         lbls = []
         if not self.vae:
             self.u.err_p(["plot_node_correlation: You haven't run the VAE yet! "
@@ -197,13 +233,9 @@ class Vis:
             padj_cols.append(f'Node {i + 1} padj')
         heatmap_df['labels'] = lbls
 
-        annot_values = heatmap_df[padj_cols].values
-        heatmap = Heatmap(heatmap_df, lbls, 'labels',
-                          title=f'{title}',
-                          cluster_cols=cluster_cols, cluster_rows=cluster_rows,
-                          vmin=vmin, vmax=vmax, cmap=cmap, figsize=self.fig_size
-                          )
-        set_style(self.fig_size, self.label_font_size)
+        # annot_values = heatmap_df[padj_cols].values
+        set_style()
+        heatmap = Heatmap(heatmap_df, lbls, 'labels', config=config)
         heatmap.plot()
         if save_fig:
             plt.savefig(os.path.join(output_dir, f'Heatmap_{title.replace(" ", "_")}.{fig_type}'))
@@ -211,10 +243,21 @@ class Vis:
             plt.show()
         return heatmap_df
 
-    def plot_node_feature_correlation(self, df: pd.DataFrame, row_id: str, vae_data=None, columns=None, output_dir="", vmin=-1, vmax=1, fig_type='svg',
-                                cluster_cols=True, cluster_rows=True, print_vals=True, encoding_type="z", cmap='RdBu_r',
-                                 title="RCM heatmap of node feature correlation", show_plt=False, save_fig=True):
-        set_style(self.fig_size, self.label_font_size)
+    def plot_node_feature_correlation(self, df: pd.DataFrame, row_id: str, vae_data=None, columns=None, output_dir="",
+                                      fig_type='svg', print_vals=True, encoding_type="z",
+                                      title="RCM heatmap of node feature corr", show_plt=False, save_fig=True,
+                                      user_config=None):
+        set_style()
+        config = {
+            'figsize': (4, 4),
+            'cluster_cols': True,
+            'cluster_rows': True,
+            'vmin': -1,
+            'vmax': 1,
+            'cmap': 'RdBu_r',
+            'title': title,
+        }
+        config = self.__merge_config(config, user_config)
         heatmap_df = pd.DataFrame()
         lbls, cols = [], []
         columns = columns if columns is not None else [c for c in df.columns if c != row_id]
@@ -250,45 +293,49 @@ class Vis:
             padj_cols.append(f'Node {i + 1} padj')
             first = False
         heatmap_df['labels'] = lbls
-        annot_values = heatmap_df[padj_cols].values
-        heatmap = Heatmap(heatmap_df, cols, 'labels',
-                          title=f'{title}',
-                          cluster_cols=cluster_cols, cluster_rows=cluster_rows,
-                          vmin=vmin, vmax=vmax, figsize=(3, 2))
+        # annot_values = heatmap_df[padj_cols].values
+        set_style()
+        heatmap = Heatmap(heatmap_df, cols, 'labels', config=config)
         heatmap.plot()
         if save_fig:
             plt.savefig(os.path.join(output_dir, f'Heatmap_{title.replace(" ", "_")}.{fig_type}'))
         if show_plt:
             plt.show()
-        set_style(self.fig_size, self.label_font_size)
         return heatmap_df
 
-    def plot_node_hists(self, nbins=20, ymax=None, output_dir="", c="grey", fig_type='svg', method="z",
-                        show_plt=False, save_fig=True, title=""):
+    def plot_node_hists(self, title="Hist", output_dir="", fig_type='svg', method="z", show_plt=False, save_fig=True,
+                        user_config=None):
         if not self.vae:
             self.u.err_p(["plot_node_correlation: You haven't run the VAE yet! "
                           "Please run compute_vae before attempting to plot."])
             return
-        plt.clf()
+        config = {
+            'figsize': (4, 4),
+            'nbins': 20,
+            'c': "grey",
+            'max_y': None,
+            'title': title
+        }
+        config = self.__merge_config(config, user_config)
+        set_style()
         data = self.vae.get_encoded_data(method)
         num_nodes = len(data[0])
         for i in range(0, num_nodes):
-            set_style(self.fig_size, self.label_font_size)
+            set_style()
             tmp_df = pd.DataFrame()
             tmp_df[f'Node {i + 1}'] = data[:, i]
-            hist = Histogram(tmp_df, f'Node {i + 1}', title=f'{title} Node {i + 1}', bins=nbins, max_y=ymax, colour=c)
-            hist.load_style(self.style)
+            config['title'] = f'{title} Node {i + 1}'
+            hist = Histogram(tmp_df, f'Node {i + 1}', config=config)
             hist.plot()
 
             if save_fig:
-                plt.savefig(os.path.join(output_dir, f'{title}_Hist_Node-{i + 1}.{fig_type}'))
+                plt.savefig(os.path.join(output_dir, f'{title}_Node-{i + 1}.{fig_type}'))
             if show_plt:
                 plt.show()
             plt.clf()
 
     @staticmethod
     def get_row_idxs(df, row_id, row_labels, row_values):
-
         row_idxs = {}
         for m in row_labels:
             row_idxs[m] = []
@@ -307,10 +354,18 @@ class Vis:
             row_idxs_lst.append(row_idxs.get(g))
         return row_idxs_lst
 
-    def plot_feature_scatters(self, df: pd.DataFrame, row_id: str, vae_data=None, columns=None, output_dir="", fig_type='svg',
-                              vmin=None, vmax=None,
-                              title="latent space", show_plt=False, save_fig=True, encoding_type="z", angle_plot=0, cmap='RdBu_r'):
-        set_style(self.fig_size, self.label_font_size)
+    def plot_feature_scatters(self, df: pd.DataFrame, row_id: str, vae_data=None, columns=None, output_dir="",
+                              fig_type='svg', title="latent space", show_plt=False, save_fig=True, encoding_type="z",
+                              angle_plot=90, user_config=None):
+        set_style()
+        config = {
+            'figsize': (4, 4),
+            'angle_plot': angle_plot,
+            'c': "grey",
+            'max_y': None,
+            'title': title
+        }
+        config = self.__merge_config(config, user_config)
         columns = columns if columns is not None else [c for c in df.columns if c != row_id]
         if not self.vae:
             self.u.err_p(["plot_scatters: You haven't run the VAE yet! "
@@ -338,27 +393,37 @@ class Vis:
             vis_df[f'Dim. 2'] = data[:, 1]
             if num_nodes == 2:
                 scatter = Scatterplot(vis_df, f'Dim. 1', f'Dim. 2', f'{column} {title}', f'Dim. 1', f'Dim. 2',
-                                      colour=pd.to_numeric(df[column].values), )
+                                      colour=pd.to_numeric(df[column].values), config=config)
             else:
                 vis_df[f'Dim. 3'] = data[:, 2]
-                scatter = Scatterplot(vis_df, f'Dim. 1', f'Dim. 2', f'{column} {title}', f'Dim. 1', f'Dim. 2', z=f'Dim. 3',
-                                      colour=pd.to_numeric(df[column].values))
+                scatter = Scatterplot(vis_df, f'Dim. 1', f'Dim. 2', f'{column} {title}', f'Dim. 1', f'Dim. 2',
+                                      z=f'Dim. 3', colour=pd.to_numeric(df[column].values), config=config)
             ax = scatter.plot()
             if save_fig:
                 plt.savefig(os.path.join(output_dir, f'Scatter_{column}_{title.replace(" ", "_")}.{fig_type}'))
-            if show_plt:
-                plt.show()
             if angle_plot:
                 # Plot different angles
                 for ii in range(0, 360, angle_plot):
                     ax.view_init(elev=10., azim=ii)
                     plt.savefig(os.path.join(output_dir, f'Scatter_{column}_{title.replace(" ", "_")}_{ii}.{fig_type}'))
+            if show_plt:
+                plt.show()
 
     def plot_values_on_scatters(self, df: pd.DataFrame, row_id: str, row_labels: list, row_values: list, vae_data=None,
-                                output_dir="", fig_type='svg', title="RCM genes on latent space", plt_bg=False,
-                                show_plt=False,
-                                save_fig=True, color_map=None, angle_plot=None, encoding_type="z"):
-        set_style(self.fig_size, self.label_font_size)
+                                output_dir="", fig_type='svg', title="RCM genes on latent space", show_plt=False,
+                                color_map=None, save_fig=True, angle_plot=None, encoding_type="z", user_config=None):
+        set_style()
+        config = {
+            'figsize': (4, 4),
+            'angle_plot': angle_plot,
+            'c': "grey",
+            'max_y': None,
+            'title': title,
+            'vmin': None,
+            'vmax': None,
+            'plt_bg': False
+        }
+        config = self.__merge_config(config, user_config)
         marker_idxs = self.get_row_idxs(df, row_id, row_labels, row_values)
 
         if vae_data is None:
@@ -394,30 +459,26 @@ class Vis:
         vis_df[f'Dim. 1'] = data[:, 0]
         vis_df[f'Dim. 2'] = data[:, 1]
         if num_nodes == 2:
-            scatter = Scatterplot(vis_df, f'Dim. 1', f'Dim. 2', f'{title}', f'Dim. 1', f'Dim. 2')
+            scatter = Scatterplot(vis_df, f'Dim. 1', f'Dim. 2', f'{title}', f'Dim. 1', f'Dim. 2', config=config)
             ax = scatter.plot_groups_2D(row_labels, marker_idxs, color_map, alpha_bg=0.1)
-            plt.savefig(os.path.join(output_dir, f'Scatter-genes_{title.replace(" ", "_")}.{fig_type}'))
         else:
             vis_df[f'Dim. 3'] = data[:, 2]
-            scatter = Scatterplot(vis_df, f'Dim. 1', f'Dim. 2', f'{title}', f'Dim. 1', f'Dim. 2', z=f'Dim. 3')
+            scatter = Scatterplot(vis_df, f'Dim. 1', f'Dim. 2', f'{title}', f'Dim. 1', f'Dim. 2', z=f'Dim. 3',
+                                  config=config)
             ax = scatter.plot_groups_3D(row_labels, marker_idxs, color_map, alpha_bg=0.1)
+        if save_fig:
             plt.savefig(os.path.join(output_dir, f'Scatter-genes_{title.replace(" ", "_")}.{fig_type}'))
-        if show_plt:
-            plt.show()
         if angle_plot:
             # Plot different angles
             for ii in range(0, 360, angle_plot):
                 ax.view_init(elev=10., azim=ii)
                 plt.savefig(os.path.join(output_dir, f'Scatter-genes_{title.replace(" ", "_")}_{ii}.{fig_type}'))
-        set_style(self.fig_size, self.label_font_size)
+        if show_plt:
+            plt.show()
+        set_style()
         return ax
 
 
-def set_style(figsize, label_font_size):
+def set_style():
     # plot
     plt.clf()
-    font = {'family': 'normal', 'size': 6}
-    matplotlib.rc('font', **font)
-    sns.set(rc={'figure.figsize': figsize, 'font.family': 'sans-serif',
-                'font.sans-serif': 'Arial', 'font.size': label_font_size}, style="ticks")
-    rcParams['figure.figsize'] = (2, 2)
\ No newline at end of file
